/*
 * Created on Jan 31, 2005

 */
package com.ibm.gre.samples.collectionhelpers;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.ArrayList;

import com.ibm.gre.engine.util.AttributeControlMask;
import com.ibm.gre.engine.util.AttributeDataTypes;
import com.ibm.gre.engine.util.ProfileControlType;
import com.ibm.gre.engine.util.XMLUtil;
import com.ibm.gre.engine.util.RecordManagerException;

/**
 * The IRMFields class encapsulates a Fields element from
 * a Records manager Complex Fields Collection returned by one of
 * the TemplateControllerEJB methods.
 * This class can be used to help get and set values for
 * Adding, Editing and Displaying Records Manager Components.
 * See the documentation for the TemplateControllerEJB interface
 * contained in the com.ibm.gre.engine.ejb.facade package in the
 * IBM DB2 Records Manager API Reference for examples and explanations
 * of methods that will return or accept Complex Fields Collection XML
 * values. 
 */
public class IRMFields {
	
	/**
	 * Constructor
	 * @param xml A Complex Fields Collection String value.
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	public IRMFields(String xml)throws RecordManagerException, Exception
	{
		m_fields = new ArrayList();
		Document doc = XMLUtil.getDocument(xml);
		m_root = doc.getDocumentElement();
		String xpath = "./Field";
		NodeList fields = XMLUtil.getNodeListByXPath(m_root,xpath);
		for(int i = 0; i < fields.getLength(); i++)
		{
			IRMField irm = new IRMField((Element)fields.item(i));
				
			//Get the File Plan Component Definition ID from one of the fields
			if(m_nFlPlnCmpntDefID == 0)
			{
				m_nFlPlnCmpntDefID = irm.getFlPlnCmpntDefID();
			}
				
			//Always set the visability of RowVrsn to false;
			//User can re-set to true if they wish to see this
			if(irm.getFlPlnCmpntAttrDefColNm() == "RowVrsn")
			{
				irm.setIsVisible(false);
			}
			if(irm.getFlPlnCmpntAttrDefColNm() == "Profile_Options")
			{
				m_Profile_Options = irm;
			}
			else if(irm.getFlPlnCmpntAttrDefColNm() == "Default_Options")
			{
				m_Default_Options = irm;
			}
			else
			{
				m_fields.add(irm);
			}
		}
	}

	/**
	 * The ID of the Component Definition type that the Fields
	 * class reprepresents.
	 * @return int
	 */
	public int getFlPlnCmpntDefID()
	{
		return m_nFlPlnCmpntDefID;
	}

	/**
	 * The number of Field objects that the
	 * class contains.
	 * @return int
	 */
	public int getFieldsCount()
	{
		return m_fields.size();
	}

	/**
	 * The number of Field objects that have
	 * been specified to be displayed.
	 * @return int
	 */
	public int getDisplayCount()
	{
		int count = 0;
		for(int i = 0; i < m_fields.size(); i++)
		{
			IRMField field = (IRMField)m_fields.get(i);
			if(field.getIsVisible()) count++;
		}
		return count;
	}

	/**
	 * Get the IRMField object for the specified index.
	 * @param index The integer value from 0 to n indicating the IRMField object to return.
	 * @return An IRMField object containing attributes and values.
	 */
	public IRMField getField(int index)
	{
		return (IRMField)m_fields.get(index);
	}

	/**
	 * Gets the IRMField object for the specified FlPlnCmpntAttrDefColNm. 
	 * @param strFlPlnCmpntAttrDefColNm File Plan Attribute Definition Column Name.
	 * @return An IRMField object containing attributes and values.
	 * @throws Exception
	 */
	public IRMField getFieldByFlPlnCmpntAttrDefColNm(String strFlPlnCmpntAttrDefColNm)
	throws Exception
	{
		for(int i = 0; i < m_fields.size(); i++)
		{
			IRMField field = (IRMField)m_fields.get(i);
			if(strFlPlnCmpntAttrDefColNm == field.getFlPlnCmpntAttrDefColNm())
			{
				return field;			
			}
		}
		throw new Exception("Could not find Field: " + strFlPlnCmpntAttrDefColNm);
	}

	/**
	 * Set the value of the underlying Field Element for the Component
	 * Attribute specified by the FlPlnCmpntAttrDefColNm.
	 * @param strFlPlnCmpntAttrDefColNm File Plan Attribute Definition Column Name.
	 * @param val The value to assign to the specified Field.
	 * @return Returns true if the value was successfully set.
	 * @throws Exception
	 */
	public boolean setFieldByFlPlnCmpntAttrDefColNm(String strFlPlnCmpntAttrDefColNm, String val)
	throws Exception
	{
		for(int i = 0; i < m_fields.size(); i++)
		{
			IRMField field = (IRMField)m_fields.get(i);
			if(strFlPlnCmpntAttrDefColNm == field.getFlPlnCmpntAttrDefColNm())
			{
				field.setValue(val);
				return true; //TODO add validation here			
			}
		}
		throw new Exception("Could not find Field: " + strFlPlnCmpntAttrDefColNm);
	}

	/**
	 * Set the visability of a specified Field.
	 * @param strFlPlnCmpntAttrDefColNm File Plan Attribute Definition Column Name.
	 * @param makeVisable Setting this value to false indicates the field should not be displayed.
	 * @throws Exception
	 */
	public void setVisabilityByFlPlnCmpntAttrDefColNm(String strFlPlnCmpntAttrDefColNm, boolean makeVisable)
	throws Exception
	{
		for(int i = 0; i < m_fields.size(); i++)
		{
			IRMField field = (IRMField)m_fields.get(i);
			if(strFlPlnCmpntAttrDefColNm == field.getFlPlnCmpntAttrDefColNm())
			{
				field.setIsVisible(makeVisable);
			}
		}
	}

	/**
	 * This is a convenience method for setting the visability
	 * attribute to Security Fields, indicating they should
	 * not be displayed. Host applications that do not use Security
	 * Fields can use this method.
	 * @throws Exception
	 */
	public void hideSecurityFields() throws Exception
	{
		setVisabilityByFlPlnCmpntAttrDefColNm("SecClsID",false);
		setVisabilityByFlPlnCmpntAttrDefColNm("SecClsCommonOwnerID",false);
		setVisabilityByFlPlnCmpntAttrDefColNm("SecClsInfo",false);
	}

	/**
	 * Sets a flag indicating whether a specified field should be include
	 * in values for Add or update.
	 * @param strFlPlnCmpntAttrDefColNm File Plan Attribute Definition Column Name.
	 * @param active Set this value to false to indicate the field should not be used.
	 * @throws Exception
	 */
	public void setActiveStateByFlPlnCmpntAttrDefColNm(String strFlPlnCmpntAttrDefColNm, boolean active)
	throws Exception
	{
		for(int i = 0; i < m_fields.size(); i++)
		{
			IRMField field = (IRMField)m_fields.get(i);
			if(strFlPlnCmpntAttrDefColNm == field.getFlPlnCmpntAttrDefColNm())
			{
				field.setIsActive(active);
			}
		}
	}

	/**
	 * A convenience method for in-activating security fields.
	 * Host applications that do not use security can use this method.
	 * @throws Exception
	 */
	public void inactivateSecurityFields() throws Exception
	{
		setActiveStateByFlPlnCmpntAttrDefColNm("SecClsID",false);
		setActiveStateByFlPlnCmpntAttrDefColNm("SecClsCommonOwnerID",false);
		setActiveStateByFlPlnCmpntAttrDefColNm("SecClsInfo",false);
	}

	/**
	 * This method returns a properly formatted XML
	 * String that can be used as input to calls to the
	 * Records Manager TemplateControllerEJB methods
	 * addNewFilePlanComponent or addNewSystemComponent.
	 * Can also be used as input to the Records Manager
	 * Component Update methods.
	 * @return Correctly formatted XML String.
	 * @throws RecordManagerException
	 */
	public String getXMLForAdd() throws RecordManagerException
	{
		Document doc = XMLUtil.getDocument("<Fields/>");
		Element fields = doc.getDocumentElement();
		for(int i = 0; i < m_fields.size(); i++)
		{
			IRMField field = (IRMField)m_fields.get(i);
			if(field.getIsActive())
			{
				fields.appendChild(doc.importNode(field.getFieldForAdd(),true));
			}
		}
		return XMLUtil.getXML(fields);
	}

	/**
	 * This method returns the number of characters
	 * contained in the longest Caption in the Fields
	 * Collection.
	 * @return String
	 * @throws Exception
	 */
	public String getMaxCaption() throws Exception
	{
		if(m_nMaxCaptionLength == 0)
		{
			for(int i = 0; i < m_fields.size(); i++)
			{
				IRMField field = (IRMField)m_fields.get(i);
				int len = field.getCaption().length(); 
				if(len > m_nMaxCaptionLength)
				{
					m_nMaxCaptionLength = len;
					m_nMaxCaptionIndex = i;
				}
			}
		}
		IRMField irm = (IRMField)m_fields.get(m_nMaxCaptionIndex);
		return irm.getCaption();
	}
	//Private Member Variables
	private Element m_root;
	private ArrayList m_fields;
	private int m_nMaxCaptionLength = 0;
	private int m_nMaxCaptionIndex = 0;
	private int m_nFlPlnCmpntDefID = 0;

	//Profile and Default data
	IRMField m_Profile_Options = null;
	IRMField m_Default_Options = null;
}
