/*
 * Created on Jan 31, 2005
 */
package com.ibm.gre.samples.collectionhelpers;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.XMLUtil;

/**
 * Encapsulates a Records Manager Item List.
 * Provides methods for getting attribute values.
 * Also methods for removing specified Items. 
 */
public class IRMList {
	
	/**
	 * Constructor
	 * @param xmlList A Records Manager List Item type XML String.
	 * See any Records Manager List method for an example of this type of XML.
	 * @param strPrimaryKeyAttributeName The name of the attribute that you wish
	 * to consider as the "Primary Key" for this List.
	 * The value of this attribute is returned by the PrimaryKey getter method.
	 * @param strCaptionAttributeName The name of the attribute that you wish
	 * to consider as the "Caption" for this list.
	 * The value of this attribute is returned by the Caption getter method.
	 * @throws RecordManagerException
	 */
	public IRMList(String xmlList, String strPrimaryKeyAttributeName, String strCaptionAttributeName)
	throws RecordManagerException
	{
		m_strPrimaryKeyAttributeName = strPrimaryKeyAttributeName;
		m_strCaptionAttributeName = strCaptionAttributeName;
			
		Document doc = XMLUtil.getDocument(xmlList);
		m_root = doc.getDocumentElement();
		m_items = m_root.getElementsByTagName("Item");
	}

	/**
	 * Returns the current value for the underlying XML for this List.
	 * @return String
	 */
	public String getXML()
	{
		return XMLUtil.getXML(m_root);
	}

	/**
	 * Returns the number of Item elements in the List.
	 */
	public int getCount()
	{
		return m_items.getLength();
	}

	/**
	 * Returns the "Primary Key" value for the specified Item.
	 * The attribute that you wish to designate as the "Primary Key"
	 * is set in the constructor for this object.
	 * @param index The index of specifying the Item to return the "Primary Key" for.
	 * @return The value of the "Primary Key".
	 */
	public String getPrimaryKey(int index) throws Exception
	{
		return getVal(index, m_strPrimaryKeyAttributeName, "");
	}

	/**
	 * Returns the "Caption" value for the specified Item.
	 * The attribute that you wish to designate as the "Caption"
	 * is set in the constructor for this object.
	 * @param index The index of specifying the Item to return the "Caption" for.
	 * @return The value for the "Caption"
	 * @throws Exception
	 */
	public String getCaption(int index) throws Exception
	{
		return getVal(index, m_strCaptionAttributeName, "");
	}

	/**
	 * Gets the value for the named attribute at the specified index.
	 * @param index The index of the Item to return the attribute value for.
	 * @param attrName The name of the attribute to return the value for.
	 * @return Value of the specified attribute.
	 * @throws Exception
	 */
	public String getAttributeValue(int index, String attrName) throws Exception
	{
		Node node = m_items.item(index).getAttributes().getNamedItem(attrName);
		if(node == null)
		{
			throw new Exception("Invalid Attribute: " + attrName);
		}
		return XMLUtil.getValue(node);
	}

	/**
	 * Removes the Item element at the specified index.
	 * @param index The index of the Item element to remove.
	 */
	public void removeItem(int index)
	{
		m_root.removeChild(m_items.item(index));
	}

	private String getVal(int index, String attrName, String defaultVal)
	throws Exception
	{
		Node node = m_items.item(index).getAttributes().getNamedItem(attrName);
		if(node != null)
		{
			String temp = XMLUtil.getValue(node);
			if(temp.length() > 0)
			{
				defaultVal = temp;
			}
		}
		return defaultVal;
	}

	private NodeList m_items;
	private Element m_root;
	private String m_strPrimaryKeyAttributeName;
	private String m_strCaptionAttributeName;

}
